.pragma library
/**
*  Version of the response data format.
*  Increase this number to force a refresh.
*/
var RESPONSE_DATA_VERSION = 1;

/**
* Helper functions
*/
function calcFahrenheit(celsius) {
        return celsius * 1.8 + 32;
}
function calcMph(kmh) {
        return kmh*0.62137;
}

var OpenWeatherMapApi = (function() {
    /**
      provides neccessary methods for requesting and preparing data from OpenWeatherMap.org
    */
    var _baseUrl = "http://api.openweathermap.org/data/2.5/";
    //
    function _buildSearchResult(request, data) {
        var searchResult = {
            locations: [],
            request: request
        }
        if(data.cod === "200" && data.list) {
            data.list.forEach(function(r) {
               searchResult.locations.push({
                    service: "openweathermap",
                    service_id: r.id,
                    name: r.name,
                    coord: r.coord,
                    country: (r.sys && r.sys.country) ? r.sys.country : ""
                });
            })
        }
        return searchResult;
    }
    //
    function _buildDataPoint(data) {
        // TODO add snow or rain data
        var result = {
            timestamp: data.dt,
            date: (data.date !== undefined) ? data.date : ((data.dt_txt !== undefined) ? data.dt_txt: null),
            metric: {
                temp:data.main.temp,
                temp_min: data.main.temp_min,
                temp_max: data.main.temp_max
            },
            imperial: {
                temp: calcFahrenheit(data.main.temp),
                temp_min: calcFahrenheit(data.main.temp_min),
                temp_max: calcFahrenheit(data.main.temp_max)
            },
            humidity: data.main.humidity,
            pressure: data.main.pressure,
            wind_speed_kmh: data.wind.speed,
            wind_speed_mph: calcMph(data.wind.speed),
            wind_deg: data.wind.deg,
            condition: data.weather[0]
        };
        if(data.id !== undefined) {
            result["service"] = "openweathermap";
            result["service_id"] =  data.id;

        }
        return result;
    }
    //
    function _buildCurrentCondition(request, data) {
        var currentResult = {
            results: _buildDataPoint(data),
            request: request
        }
        return currentResult;
    }
    //
    function _buildForecast(request, data) {
        var forecastResult = {
            results: [],
            request: request
        }
        data.list.forEach(function(f) {
            forecastResult.results.push(_buildDataPoint(f));
        });
        return forecastResult;
    }
    //
    function _buildDailyForecast(request, data) {
        var forecastResult = {
            results: [],
            request: request
        }
        data.list.forEach(function(f) {
            // TODO date_txt
            forecastResult.results.push({
                timestamp: f.dt,
                metric: {
                    day: f.temp.day,
                    min: f.temp.min,
                    max: f.temp.max,
                    night: f.temp.night,
                    eve: f.temp.eve,
                    morn: f.temp.morn
                },
                imperial: {
                    day: calcFahrenheit(f.temp.day),
                    min: calcFahrenheit(f.temp.min),
                    max: calcFahrenheit(f.temp.max),
                    night: calcFahrenheit(f.temp.night),
                    eve: calcFahrenheit(f.temp.eve),
                    morn: calcFahrenheit(f.temp.morn)
                },
                pressure: f.pressure,
                humidity: f.humidity,
                condition: f.weather[0],
                wind_speed: f.speed,
                wind_deg: f.deg
            });
        });
        return forecastResult;
    }
    //
    return {
        //
        getSearchByNameRequest: function(params) {
            var request = {
                type: "search",
                url: _baseUrl+"find?q="+encodeURIComponent(params.name)+"&units="+params.units,
                formatter: _buildSearchResult
            }
            return request;
        },
        //
        getSearchByPointRequest: function(params) {
            var url = _baseUrl+"find?lat="+encodeURIComponent(params.coords.lat)
                    +"&lon="+encodeURIComponent(params.coords.lon)+"&units="+params.units,
                request = {
                    type: "search",
                    url: url,
                    formatter: _buildSearchResult
                };
            return request;
        },
        //
        getCurrentConditionRequest: function(params) {
            var request = {
                type: "current",
                url: _baseUrl + "weather?id="+params.location.service_id+"&units="+params.units,
                formatter: _buildCurrentCondition
            }
            return request;
        },
        //
        getForecastRequest: function(params) {
            var request = {
                type: "forecast",
                url: _baseUrl + "forecast?id="+params.location.service_id+"&units="+params.units,
                formatter: _buildForecast
            }
            return request;
        },
        //
        getDailyForecastRequest: function(params) {
            var request = {
                type: "daily",
                url: _baseUrl + "forecast/daily?id="+params.location.service_id
                     +"&cnt=10&units="+params.units,
                formatter: _buildDailyForecast
            }
            return request;
        }
    }

})();

var WeatherApi = (function(_services) {
    /**
      proxy for requesting weather apis, the passed _services are providing the respective api endpoints
      and formatters to build a uniform response object
    */
    function _getService(name) {
        if(_services[name] !== undefined) {
            return _services[name];
        } 
        return _services["openweathermap"];
    }
    //
    function _sendRequest(request, onSuccess, onError) {        
            var xmlHttp = new XMLHttpRequest();
            if (xmlHttp) {
                console.log(request.url);
                xmlHttp.open('GET', request.url, true);
                xmlHttp.onreadystatechange = function () {
                    try {
                        if (xmlHttp.readyState == 4) {
                            //console.log(xmlHttp.responseText);
                            var json = JSON.parse(xmlHttp.responseText);
                            if(xmlHttp.status === 200) {
                                var result = request.formatter(request,json);
                                onSuccess(result);
                            } else {
                                onError({
                                    msg: "wrong response http code, got "+xmlHttp.status,
                                    request: request
                                });
                            }
                        }
                    } catch (e) {
                        onError({msg: "wrong response data format", request: request});
                    }
                };
                xmlHttp.send(null);
            }
    }
    //
    return  {
        getForecast: function(params, onSuccess, onError) {
            var request = _getService().getForecastRequest(params);
            _sendRequest(request, onSuccess, onError);
        },
        //
        getDailyForecast: function(params, onSuccess, onError) {
            var request = _getService().getDailyForecastRequest(params);
            _sendRequest(request, onSuccess, onError);
        },
        //
        getCurrentCondition: function(params, onSuccess, onError) {
            var request = _getService().getCurrentConditionRequest(params);
            _sendRequest(request, onSuccess, onError);
        },
        //
        searchLocationByName: function(params, onSuccess, onError) {
            var request = _getService().getSearchByNameRequest(params);
            _sendRequest(request, onSuccess, onError);
        },
        //
        searchLocationByPoint: function(params, onSuccess, onError) {
            var request = _getService().getSearchByPointRequest(params);
            _sendRequest(request, onSuccess, onError);
        },
        //
        search: function(mode, params, onSuccess, onError) {
            var searchFunc = (mode === "point") ? this.searchLocationByPoint: this.searchLocationByName,
                retryHandler = (function(err) {
                    console.log("search retry of "+err.request.url);
                    searchFunc(params, onSuccess, onError);
            });
            searchFunc(params, onSuccess, retryHandler);
        },
        //
        getLocationData: function(params, onSuccess, onError) {
            var retryMap = {
                current:this.getCurrentCondition,
                daily:this.getDailyForecast,
                forecast:this.getForecast
            },
            response = {
                location: params.location,
                db: (params.db) ? params.db : null,
                format: RESPONSE_DATA_VERSION
            },
            addDataToResponse = (function(data) {
                response[data.request.type] = data;
                if(response["current"] !== undefined
                        //&& response["forecast"] !== undefined
                            && response["daily"] !== undefined) {
                    onSuccess(response);
                }
            }),
            onErrorHandler = (function(err) {
                onError(err);
            }),
            retryHandler = (function(err) {
                console.log("retry of "+err.request.url);
                var retryFunc = retryMap[err.request.type];
                retryFunc(params, addDataToResponse, onErrorHandler);
            })

            this.getCurrentCondition(params, addDataToResponse, retryHandler);
            //this.getForecast(params, addDataToResponse, retryHandler);
            this.getDailyForecast(params, addDataToResponse, retryHandler);
        }
    }
})({
    "openweathermap": OpenWeatherMapApi
});

/**
*  following WorkerScript handles the data requests against the weather API.
*  "message" requires a "params" property with the required params to perform
*  the API call and an "action" property, which will be added also to the response.
*/
if(typeof WorkerScript != "undefined") {
    WorkerScript.onMessage = function(message) {
        // handles the response data
        var finished = function(result) {            
            WorkerScript.sendMessage({
                action: message.action,
                result: result
            })
        }
        // handles errors
        var onError = function(err) {
            console.log(JSON.stringify(err, null, true));
            WorkerScript.sendMessage({ 'error': err})
        }
        // keep order of locations, sort results
        var sortDataResults = function(locA, locB) {
            return locA.db.id - locB.db.id;
        }
        // perform the api calls
        if(message.action === "searchByName") {
            WeatherApi.search("name", message.params, finished, onError);
        } else if(message.action === "searchByPoint") {
            WeatherApi.search("point", message.params, finished, onError);
        } else if(message.action === "updateData") {
            var locLength = message.params.locations.length,
                locUpdated = 0,
                result = [],
                now = new Date().getTime();
            if(locLength > 0) {
                message.params.locations.forEach(function(loc) {
                    var updatedHnd = function (newData, cached) {
                            locUpdated += 1;                            
                            newData["save"] = (cached === true) ? false : true;
                            result.push(newData);
                            if(locUpdated === locLength) {
                                result.sort(sortDataResults);
                                finished(result);
                            }
                        },
                        params = {
                            location:loc.location,
                            db: loc.db,
                            units: 'metric'
                        },
                        secsFromLastFetch = (now-loc.updated)/1000;
                    //
                    if( message.params.force===true || loc.format !== RESPONSE_DATA_VERSION || secsFromLastFetch > 1800){
                        // data older than 30min, location is new or data format is deprecated
                        WeatherApi.getLocationData(params, updatedHnd, onError);
                    } else {
                        console.log("["+loc.location.name+"] returning cached data, time from last fetch: "+secsFromLastFetch)
                        updatedHnd(loc, true);
                    }
                })
            } else {
                finished(result);
            }
        }
    }
}
