# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013 Canonical
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

"""Tests for managing locations in Weather app"""

from __future__ import absolute_import

from testtools.matchers import Equals
from autopilot.matchers import Eventually

from ubuntu_weather_app.tests import WeatherTestCase, DatabaseMixin
from ubuntu_weather_app.tests.weatherdata import locations_data

class TestSettings(WeatherTestCase, DatabaseMixin):
    def setUp(self):
        self.clean_db()
        self.launch_and_quit_app()
        self.save_locations_to_storage(locations_data)
        super(TestSettings, self).setUp()
        self.assertThat(
            self.main_window.get_qml_view().visible, Eventually(Equals(True)))

    def _open_settings_sheet(self):
        """Opens the settings sheet"""
        self.main_window.open_toolbar()
        self.assertThat(self.main_window.get_toolbar().opened, Eventually(Equals(True)))
        self.main_window.click_toolbar_button("SettingsButton")

    def _move_pointer_around(self):
        """Helper method to move the pointer around, to assure selector is opened"""
        sheet = self.main_window.get_object('ComposerSheet', 'SettingsSheet')
        self.pointing_device.move_to_object(sheet)

    def _click_confirm(self):
        """Clicks the confirm button"""
        self.pointing_device.click_object(self.main_window.app.select_many('Button')[1])

    def _click_cancel(self):
        """Clicks the cancel button"""
        self.pointing_device.click_object(self.main_window.app.select_many('Button')[0])

    def _check_units(self, units):
        """Checks selected units by values from the first location tab"""
        current_temps = self.main_window.get_objects('QQuickText', 'CurrentTempText')
        if units == "imperial":
            self.assertThat(current_temps[0].text, Eventually(Equals(u'69°F')))
            self.assertThat(current_temps[1].text, Eventually(Equals(u'78°F')))
        else:
            self.assertThat(current_temps[0].text, Eventually(Equals(u'21°C')))
            self.assertThat(current_temps[1].text, Eventually(Equals(u'26°C')))

    def test_switch_scale(self):
        """Tests switching the scale in the settings"""
        # first check metric values and open the settings sheet
        self._check_units('metric')
        self._open_settings_sheet()

        # open the value selector
        units_selector = self.main_window.get_object('ValueSelector', "UnitsSelector")
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(0)))
        self.pointing_device.click_object(units_selector)

        # choose second option, fahrenheit
        self._move_pointer_around()
        imperial_option = units_selector.get_children()[3]
        self.pointing_device.click_object(imperial_option)
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(1)))
        self._click_confirm()

        # wait for reload and check the imperial values
        load_indicator = self.main_window.get_object('ActivityIndicator', 'LoadingSpinner')
        self.assertThat(load_indicator.running, Eventually(Equals(False)))
        self._check_units('imperial')

        # switch back to metric values again
        self._open_settings_sheet()
        units_selector = self.main_window.get_object('ValueSelector', "UnitsSelector")
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(1)))
        self.pointing_device.click_object(units_selector)
        self.assertThat(units_selector.expanded, Eventually(Equals(True)))

        # click celsius option
        self._move_pointer_around()
        metric_option = units_selector.get_children()[4]
        self.pointing_device.click_object(metric_option)

        # confirm
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(0)))
        self._click_confirm()

        # wait for reload and check the metric values again
        load_indicator = self.main_window.get_object('ActivityIndicator', 'LoadingSpinner')
        self.assertThat(load_indicator.running, Eventually(Equals(False)))
        self._check_units('metric')

    def test_switch_scale_cancel(self):
        """Test canceling switching scale"""
        # first check metric values and open the settings sheet
        self._check_units('metric')
        self._open_settings_sheet()

        units_selector = self.main_window.get_object('ValueSelector', "UnitsSelector")
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(0)))
        self.pointing_device.click_object(units_selector)

        # choose second option, fahrenheit
        self._move_pointer_around()
        imperial_option = units_selector.get_children()[3]
        self.pointing_device.click_object(imperial_option)
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(1)))

        # cancel and check
        self._click_cancel()
        self._check_units('metric')

